/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef PIMIENTO_BUTTONS_H
#define PIMIENTO_BUTTONS_H

#include <Arduino.h>

class PimientoButtons {

  public:
    static const uint8_t EVENT_NONE = 0;
    static const uint8_t EVENT_PRESSED = 1;
    static const uint8_t EVENT_RELEASED = 2;
    
    static const uint8_t BUTTON_STATE_PRESSED = LOW;
    static const uint8_t BUTTON_STATE_NOT_PRESSED = HIGH;

    static const unsigned long SCAN_EVERY_MILLIS = 100;

    void init(uint8_t buttonPin0);
    void init(uint8_t buttonPin0, uint8_t buttonPin1);
    void init(uint8_t buttonPin0, uint8_t buttonPin1, uint8_t buttonPin2);
    void init(uint8_t buttonPin0, uint8_t buttonPin1, uint8_t buttonPin2, uint8_t buttonPin3);
    void init(uint8_t buttonPin0, uint8_t buttonPin1, uint8_t buttonPin2, uint8_t buttonPin3, uint8_t buttonPin4);

    /* Returns the first button index for which an event happened, or -1 if no event was registered for any of the buttons */
    int8_t readStates();

    /* Gets the event for the specified button */
    uint8_t getEvent(uint8_t buttonIndex);
    
    /* Gets the event for the specified button and clears the event */
    uint8_t consumeEvent(uint8_t buttonIndex);

  private:

    static const uint8_t BUTTON_COUNT = 5;

    uint8_t buttonPins[BUTTON_COUNT];
    uint8_t oldButtonState[BUTTON_COUNT];
    uint8_t buttonEvent[BUTTON_COUNT];
    uint8_t oldButtonEvent[BUTTON_COUNT];

    /* Clears the event and states for the specified button */
    uint8_t clearState(uint8_t buttonIndex);

    bool isConfiguredButton(uint8_t buttonIndex);

};

extern PimientoButtons buttons;

#endif /* PIMIENTO_BUTTONS_H */
